package com.couplet.gateway.service.impl;

import com.google.code.kaptcha.Producer;
import com.couplet.common.core.constant.CacheConstants;
import com.couplet.common.core.constant.Constants;
import com.couplet.common.core.exception.CaptchaException;
import com.couplet.common.core.utils.StringUtils;
import com.couplet.common.core.utils.sign.Base64;
import com.couplet.common.core.utils.uuid.IdUtils;
import com.couplet.common.core.domain.Result;
import com.couplet.common.redis.service.RedisService;
import com.couplet.gateway.config.properties.CaptchaProperties;
import com.couplet.gateway.model.resp.CaptchaCodeResp;
import com.couplet.gateway.service.ValidateCodeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.FastByteArrayOutputStream;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.concurrent.TimeUnit;

/**
 * 验证码实现处理
 *
 * @author couplet
 */
@Service
public class ValidateCodeServiceImpl implements ValidateCodeService {
    @Resource(name = "captchaProducer")
    private Producer captchaProducer;

    @Resource(name = "captchaProducerMath")
    private Producer captchaProducerMath;

    @Autowired
    private RedisService redisService;

    @Autowired
    private CaptchaProperties captchaProperties;

    /**
     * 生成验证码
     */
    @Override
    public Result createCaptcha () throws IOException, CaptchaException {
        boolean captchaEnabled = captchaProperties.getEnabled();
        CaptchaCodeResp.CaptchaCodeRespBuilder respBuilder = CaptchaCodeResp.builder()
                .captchaEnabled(captchaEnabled);
        if (!captchaEnabled) {
            return Result.success(respBuilder);
        }

        // 保存验证码信息
        String uuid = IdUtils.simpleUUID();
        String verifyKey = CacheConstants.CAPTCHA_CODE_KEY + uuid;

        String capStr = null, code = null;
        BufferedImage image = null;

        String captchaType = captchaProperties.getType();
        // 生成验证码
        if ("math".equals(captchaType)) {
            String capText = captchaProducerMath.createText();
            capStr = capText.substring(0, capText.lastIndexOf("@"));
            code = capText.substring(capText.lastIndexOf("@") + 1);
            image = captchaProducerMath.createImage(capStr);
        } else if ("char".equals(captchaType)) {
            capStr = code = captchaProducer.createText();
            image = captchaProducer.createImage(capStr);
        }

        redisService.setCacheObject(verifyKey, code, Constants.CAPTCHA_EXPIRATION, TimeUnit.MINUTES);
        // 转换流信息写出
        FastByteArrayOutputStream os = new FastByteArrayOutputStream();
        try {
            ImageIO.write(image, "jpg", os);
        } catch (IOException e) {
            return Result.error(e.getMessage());
        }

        CaptchaCodeResp captchaCodeResp = respBuilder.uuid(uuid)
                .img(Base64.encode(os.toByteArray()))
                .build();
        return Result.success(captchaCodeResp);
    }

    /**
     * 校验验证码
     */
    @Override
    public void checkCaptcha (String code, String uuid) throws CaptchaException {
        if (StringUtils.isEmpty(code)) {
            throw new CaptchaException("验证码不能为空");
        }
        if (StringUtils.isEmpty(uuid)) {
            throw new CaptchaException("验证码已失效");
        }
        String verifyKey = CacheConstants.CAPTCHA_CODE_KEY + uuid;
        String captcha = redisService.getCacheObject(verifyKey);
        redisService.deleteObject(verifyKey);

        if (!code.equalsIgnoreCase(captcha)) {
            throw new CaptchaException("验证码错误");
        }
    }
}
