package com.couplet.business.server.controller;

import com.couplet.business.server.service.VehicleService;
import com.couplet.common.core.domain.Result;
import com.couplet.common.core.web.controller.BaseController;
import com.couplet.common.domain.Vehicle;
import com.couplet.common.domain.VehicleMiddle;
import com.couplet.common.domain.request.VehicleEditParams;
import com.couplet.common.domain.request.VehicleInsertParams;
import com.couplet.common.domain.request.VehicleListParams;
import com.couplet.common.log.annotation.Log;
import com.couplet.common.log.enums.BusinessType;
import com.couplet.common.security.annotation.RequiresPermissions;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @ProjectName: five-groups-couplet
 * @Author: LiuYunHu
 * @CreateTime: 2024/3/26
 * @Description: 车辆管理
 */
@RestController
@RequestMapping("/vehicle")
public class VehicleController extends BaseController {
    @Autowired
    private VehicleService vehicleService;

    /*
     * @Author: LiuYunHu
     * @Date: 2024/3/26 21:39
     * @Description: 查询车辆列表
     * @Param: [listParams]
     * @Return: com.couplet.common.core.domain.Result
     **/
    @RequiresPermissions("couplet:vehicle:list")
    @PostMapping("/list")
    @Log(title = "车辆列表")
    public Result list(@RequestBody VehicleListParams listParams) {
        PageHelper.startPage(listParams.getPageNum(), listParams.getPageSize());

        List<Vehicle> list = vehicleService.list(listParams);

        PageInfo<Vehicle> vehiclePageInfo = new PageInfo<>(list);

        return Result.success(vehiclePageInfo);
    }

    /*
     * @Author: LiuYunHu
     * @Date: 2024/3/26 22:35
     * @Description: 通过id删除车辆
     * @Param: [vehicleId]
     * @Return: com.couplet.common.core.domain.Result
     **/
    @RequiresPermissions("couplet:vehicle:deleteById")
    @GetMapping("/deleteById/{vehicleId}")
    @Log(title = "删除车辆", businessType = BusinessType.DELETE)
    public Result<String> deleteById(@PathVariable Long vehicleId) {
        String result = vehicleService.deleteById(vehicleId);

        return Result.success(result);
    }


    /*
     * @Author: LiuYunHu
     * @Date: 2024/3/27 16:09
     * @Description: 编辑车辆
     * @Param: [editParams]
     * @Return: com.couplet.common.core.domain.Result
     **/
    @RequiresPermissions("couplet:vehicle:editById")
    @PostMapping("/editById")
    @Log(title = "编辑车辆", businessType = BusinessType.UPDATE)
    public Result<String> editById(@RequestBody VehicleEditParams editParams) {

        String result = vehicleService.editById(editParams);

        return Result.success(result);
    }

    /*
     * @Author: LiuYunHu
     * @Date: 2024/3/27 16:21
     * @Description: 新增车辆
     * @Param: [insertParams]
     * @Return: com.couplet.common.core.domain.Result
     **/
    @RequiresPermissions("couplet:vehicle:insert")
    @PostMapping("/insert")
    @Log(title = "新增车辆", businessType = BusinessType.INSERT)
    public Result<String> insert(@RequestBody @Validated VehicleInsertParams insertParams) {
        System.out.println(insertParams);
        String result = vehicleService.insert(insertParams);

        return Result.success(result);
    }


    /*
     * @Author: LiuYunHu
     * @Date: 2024/3/31 21:34
     * @Description: 获取绑定当前车辆的标识
     * @Param: [vehicleId]
     * @Return: com.couplet.common.core.domain.Result
     **/
    @RequiresPermissions("couplet:vehicle:list")
    @GetMapping("/getBindLogoById/{vehicleId}")
    public Result<List<Long>> getBindLogoById(@PathVariable("vehicleId") Long vehicleId) {

        List<Long> bindLogoById = vehicleService.getBindLogoById(vehicleId);


        return Result.success(bindLogoById);
    }


    /*
     * @Author: LiuYunHu
     * @Date: 2024/4/2 15:35
     * @Description: 通过vin码查询车辆
     * @Param: [vin]
     * @Return: com.couplet.common.core.domain.Result<java.util.List<com.couplet.vehicle.domain.Vehicle>>
     **/
    @GetMapping("/findByVIN/{vin}")
    public Result<List<Vehicle>> findByVIN(@PathVariable("vin") String vin) {
        List<Vehicle> list = vehicleService.findByVIN(vin);

        return Result.success(list);
    }




    @PostMapping("UserUnderTheVehicle/{userId}")
    public Result<List<Vehicle>> UserUnderTheVehicleList(@PathVariable(value = "userId") Long userId){
        return Result.success(vehicleService.UserUnderTheVehicleList(userId));
    }


    /*
     * @param middleId:
     * @return Result<Integer>
     * @author 付凡芮
     * @description 删除员工管理的车辆
     * @date
     */
    @DeleteMapping("/{middleId}")
    public Result<Integer> deleteVehicle(@PathVariable(value = "middleId") Long middleId){
        return Result.success(vehicleService.deleteVehicle(middleId));
    }


    @PostMapping
    public Result<Integer> addVehicle(@RequestBody VehicleMiddle vehicleMiddle){
        return Result.success(vehicleService.addVehicle(vehicleMiddle));
    }



    @PostMapping("vehicleAll")
    public Result<List<Vehicle>> VehicleManageList(){

        return Result.success(vehicleService.vehicleAll());
    }





    /*
     * @Author: LiuYunHu
     * @Date: 2024/4/4 11:28
     * @Description:  通过vin修改车辆上下线的状态
     * @Param:
     * @Return:
     **/
    @GetMapping("onOrOutLineByVIN")
    public Integer onOrOutLineByVIN(@RequestParam("params") String params) {
        String[] split = params.split(",");


        return vehicleService.onOrOutLineByVIN(split[0], Integer.parseInt(split[1]));
    }



}
