package com.couplet.business.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.couplet.business.server.mapper.VehicleMapper;
import com.couplet.business.server.service.VehicleAndLogoService;
import com.couplet.business.server.service.VehicleService;
import com.couplet.business.server.service.VehicleTypeService;
import com.couplet.common.core.domain.Result;
import com.couplet.common.core.utils.StringUtils;
import com.couplet.common.core.utils.uuid.UUID;
import com.couplet.common.domain.Vehicle;
import com.couplet.common.domain.VehicleMiddle;
import com.couplet.common.domain.VehicleType;
import com.couplet.common.domain.request.VehicleEditParams;
import com.couplet.common.domain.request.VehicleInsertParams;
import com.couplet.common.domain.request.VehicleListParams;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @ProjectName: five-groups-couplet
 * @Author: LiuYunHu
 * @CreateTime: 2024/3/26
 * @Description: 车辆服务实现类
 */

@Service
@Slf4j
public class VehicleServiceImpl extends ServiceImpl<VehicleMapper, Vehicle> implements VehicleService {
    //车辆mapper
    @Autowired
    private VehicleMapper vehicleMapper;

    //车辆类型服务
    @Autowired
    private VehicleTypeService vehicleTypeService;

    //车辆与标志关联 的mapper
    @Autowired
    private VehicleAndLogoService vehicleAndLogoService;

    /*
     * @Author: LiuYunHu
     * @Date: 2024/3/26 22:11
     * @Description: 查询列表
     * @Param: [listParams]
     * @Return: java.util.List<com.couplet.vehicle.domain.LyhVehicle>
     **/
    @Override
    public List<Vehicle> list(VehicleListParams listParams) {
        // 创建查询条件包装器
        LambdaQueryWrapper<Vehicle> queryWrapper = new LambdaQueryWrapper<>();

        // 如果车辆类型不为空，添加车辆类型作为查询条件
        if (!StringUtils.isNull(listParams.getVehicleType())) {
            queryWrapper.eq(Vehicle::getVehicleType, listParams.getVehicleType());
        }

        // 如果车辆状态不为空，添加车辆状态作为查询条件
        if (!StringUtils.isNull(listParams.getVehicleState())) {
            queryWrapper.eq(Vehicle::getVehicleState, listParams.getVehicleState());
        }

        //只查询车辆未被删除的
        queryWrapper.eq(Vehicle::getIsdelete, 0);

        // 执行查询
        List<Vehicle> list = this.list(queryWrapper);

        //调用服务，得到车辆类型
        List<VehicleType> vehicleTypes = vehicleTypeService.list();

        //将类型id一样的记录 名称进行赋值
        list.forEach(vehicle -> {
            vehicleTypes.forEach(vehicleType -> {
                if (vehicle.getVehicleType() == vehicleType.getVehicleTypeId()) {
                    vehicle.setVehicleTypeName(vehicleType.getVehicleTypeName());
                }
            });
        });


        return list;
    }

    /*
     * @Author: LiuYunHu
     * @Date: 2024/3/26 22:31
     * @Description: 通过id删除
     * @Param: [vehicleId]
     * @Return: java.lang.String
     **/
    @Override
    public String deleteById(Long vehicleId) {
        String result = "";

        UpdateWrapper<Vehicle> updateWrapper = new UpdateWrapper<>();

        updateWrapper.set("isdelete", 1)
                .eq("vehicle_id", vehicleId);

        boolean update = update(updateWrapper);

        if (!update) {
            result = "删除失败";
            Result.error(result);
        }

        result = "删除成功！";

        return result;
    }

    /*
     * @Author: LiuYunHu
     * @Date: 2024/3/27 15:21
     * @Description: 通过id进行编辑
     * @Param: [editParams]
     * @Return: java.lang.String
     **/
    @Override
    public String editById(VehicleEditParams editParams) {
        String result = "";

        if ((editParams.getLogoIds() == null || editParams.getLogoIds().isEmpty())) {
            result = "未选择电子围栏";
            Result.error(result);
        }

        UpdateWrapper<Vehicle> updateWrapper = new UpdateWrapper<>();

        //编辑车辆类型
        updateWrapper.set("vehicle_type", editParams.getVehicleType())

                //编辑电机厂商
                .set("motor_manufacturer", editParams.getMotorManufacturer())

                //编辑电池厂商
                .set("battery_manufacturer", editParams.getBatteryManufacturer())

                //编辑电机编号
                .set("motor_number", editParams.getMotorNumber())

                //编辑电池编号
                .set("battery_number", editParams.getBatteryNumber())

                //匹配车辆id
                .eq("vehicle_id", editParams.getVehicleId());


        boolean update = update(updateWrapper);

        if (!update) {
            result = "编辑失败";
            Result.error(result);
        }

        //删除掉车辆id为入参的车辆、标识的中间表数据
        int delete = vehicleAndLogoService.deleteByVehicleId(editParams.getVehicleId());

        //删除绑定之后，再添加新的绑定
        vehicleAndLogoService.vehicleBindLogo(editParams.getVehicleId(), editParams.getLogoIds());

        result = "编辑成功！";

        return result;
    }

    /*
     * @Author: LiuYunHu
     * @Date: 2024/3/27 16:34
     * @Description: 新增车辆
     * @Param: [insertParams]
     * @Return: java.lang.String
     **/
    @Override
    public String insert(VehicleInsertParams insertParams) {
        String result = "";

        if ((insertParams.getLogoIds() == null || insertParams.getLogoIds().isEmpty())) {
            result = "未选择电子围栏";
            Result.error(result);
        }


        //UUID生成17位随机字符串
        UUID uuid = UUID.randomUUID();
        String string = uuid.toString();
        String s = string.replaceAll("-", "");
        String vin = s.substring(0, 17);
//        System.out.println(vin+"   "+vin.length());

        /*
        //雪花算法生成随机数
        SnowflakeIdGenerator idGenerator = new SnowflakeIdGenerator(1, 1);
        long randomId = idGenerator.nextId();
        //转成字符串，便于切割
        String vin = "" + randomId;

        //切割，只留后17位
        vin = vin.substring(vin.length() - 17);

         */


        //创建入参对象
        Vehicle vehicle = new Vehicle();

        vehicle.setVehicleType(insertParams.getVehicleType());
        vehicle.setMotorManufacturer(insertParams.getMotorManufacturer());
        vehicle.setBatteryManufacturer(insertParams.getBatteryManufacturer());
        vehicle.setMotorNumber(insertParams.getMotorNumber());
        vehicle.setBatteryNumber(insertParams.getBatteryNumber());
        vehicle.setVin(vin);
        vehicle.setVehicleState(0);
        vehicle.setIsdelete(0);


        //执行插入操作
        int insert = vehicleMapper.insert(vehicle);

        if (insert == 0) {
            result = "新增失败";
            Result.error(result);
        }

        //获取新增的车辆id值
        //执行添加电子围栏
        int i = vehicleAndLogoService.vehicleBindLogo(vehicle.getVehicleId(), insertParams.getLogoIds());


        result = "新增成功！";

        return result;
    }


    /*
     * @Author: LiuYunHu
     * @Date: 2024/3/31 21:34
     * @Description: 获取绑定当前车辆的标识
     * @Param: [vehicleId]
     * @Return: java.util.List<java.lang.Long>
     **/
    @Override
    public List<Long> getBindLogoById(Long vehicleId) {
        List<Long> logoIds = vehicleAndLogoService.getBindLogoById(vehicleId);

        return logoIds;
    }

    /*
     * @param userId:
      * @return List<Vehicle>
     * @author 付凡芮
     * @description 根据登入人id查询管理车辆
     *
     */
    @Override
    public List<Vehicle> UserUnderTheVehicleList(Long userId) {
        return vehicleMapper.UserUnderTheVehicleList(userId);
    }

    @Override
    public Integer deleteVehicle(Long middleId) {
        return vehicleMapper.deleteVehicle(middleId);
    }

    @Override
    public Integer addVehicle(VehicleMiddle vehicleMiddle) {

        return vehicleMapper.addVehicle(vehicleMiddle);
    }

    @Override
    public List<Vehicle> vehicleAll() {
        return vehicleMapper.vehicleAll();
    }


    //通过vin修改车辆上下线的状态
    @Override
    public Integer onOrOutLineByVIN(String vin, int status) {

        return vehicleMapper.onOrOutLineByVIN(vin, status);
    }

    @Override
    public List<Vehicle> findByVIN(String vin) {

        //创建查询条件包装器
        LambdaQueryWrapper<Vehicle> queryWrapper = new LambdaQueryWrapper<>();

        if (!StringUtils.isEmpty(vin)) {
            queryWrapper.eq(Vehicle::getVin, vin);
        }

        //执行查询
        return this.list(queryWrapper);
    }

}
