package com.couplet.mq.config;

import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.connection.CorrelationData;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.amqp.support.converter.MessageConverter;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;


/**
 * @ProjectName: five-groups-couplet
 * @Author: LiuYunHu
 * @CreateTime: 2024/3/29
 * @Description: rabbitMQ配置类
 */

@Configuration
@Slf4j
public class RabbitMQConfig implements RabbitTemplate.ConfirmCallback, RabbitTemplate.ReturnsCallback {
    // 通过注入的方式获取队列名、交换机名和路由键
    //队列名
    @Value("queueName")
    public String queueName;

    //队列名
    public static final String FENCE_QUEUE ="fenceQueue";

    //交换机
    @Value("exchangeName")
    public String exchangeName;
    public static final String FENCE_EXCHANGE="fenceExchange";

    //路由键
    @Value("routingKey")
    public String routingKey;
    //路由键
    public static final String FENCE_ROUTINGKEY="fenceRoutingKey";

    private RabbitTemplate rabbitTemplate;

    /*
     * @Author: LiuYunHu
     * @Date: 2024/3/29 21:25
     * @Description: 创建并返回一个消息转换器，用于转换消息体。
     * @Param: []
     * @Return: 返回一个Jackson2JsonMessageConverter实例，用于JSON格式的消息转换。
     **/
    @Bean
    public MessageConverter messageConverter() {
        return new Jackson2JsonMessageConverter();
    }

    /*
     * @Author: LiuYunHu
     * @Date: 2024/3/29 21:26
     * @Description: 创建并返回一个持久化的队列。
     * @Param: []
     * @Return: 返回一个配置好的Queue实例。
     **/
    @Bean
    public Queue queue() {
        return new Queue(queueName, true);
    }
    @Bean("fenceQueue")
    public Queue queue2() {
        return new Queue(FENCE_QUEUE, true);
    }

    /*
     * @Author: LiuYunHu
     * @Date: 2024/3/29 21:26
     * @Description: 创建并返回一个直连交换机。
     * @Param: []
     * @Return: 返回一个配置好的DirectExchange实例。
     **/
    @Bean
    public DirectExchange directExchange() {
        return new DirectExchange(exchangeName);
    }

    @Bean("fenceExchange")
    public DirectExchange directExchange2() {
        return new DirectExchange(FENCE_EXCHANGE);
    }

    /*
     * @Author: LiuYunHu
     * @Date: 2024/3/29 21:27
     * @Description: 配置并返回RabbitTemplate实例，用于发送消息。
     * @Param: connectionFactory RabbitMQ连接工厂。
     * @Return: 配置好的RabbitTemplate实例。
     **/
    @Primary
    @Bean
    public RabbitTemplate rabbitTemplate(ConnectionFactory connectionFactory) {
        RabbitTemplate rabbitTempalte = new RabbitTemplate(connectionFactory);
        this.rabbitTemplate = rabbitTempalte;
        rabbitTempalte.setMessageConverter(messageConverter());
        rabbitTempalte();

        return rabbitTempalte;
    }

    /*
     * @Author: LiuYunHu
     * @Date: 2024/3/29 21:27
     * @Description: 配置RabbitTemplate的回调函数。
     * @Param: []
     * @Return: void
     **/
    public void rabbitTempalte() {
        rabbitTemplate.setConfirmCallback(this);
        rabbitTemplate.setReturnsCallback(this);
    }

    /*
     * @Author: LiuYunHu
     * @Date: 2024/3/29 21:27
     * @Description: 创建并返回一个绑定，将队列与交换机绑定，并指定路由键
     * @Param: []
     * @Return: 返回一个配置好的Binding实例
     **/
    @Bean
    public Binding binding() {
        return BindingBuilder.bind(queue()).to(directExchange()).with(routingKey);
    }

    @Bean("fenceRoutingKey")
    public Binding binding2() {
        return BindingBuilder.bind(queue2()).to(directExchange2()).with(FENCE_ROUTINGKEY);
    }

    /*
     * @Author: LiuYunHu
     * @Date: 2024/3/29 21:28
     * @Description: 消息确认回调函数。
     * 当消息被交换机成功处理后调用，或当消息未能被正确处理时调用。
     * @Param: correlationData 关联数据，用于追踪消息
     * @Param: ack 消息是否被成功处理
     * @Param: s 附加信息
     **/
    @Override
    public void confirm(CorrelationData correlationData, boolean ack, String s) {
        if (ack) {
            log.info("{}消息到达交换机", correlationData.getId());
        } else {
            log.error("{}消息丢失", correlationData.getId());
        }
    }

    /*
     * @Author: LiuYunHu
     * @Date: 2024/3/29 21:29
     * @Description: 消息返回回调函数。
     * 当消息未能被正确路由到队列时调用
     * @Param: returnedMessage 被返回的消息
     * @Return: void
     **/
    @Override
    public void returnedMessage(ReturnedMessage returnedMessage) {
        log.error("{}消息未到达队列", returnedMessage.getMessage().getMessageProperties().getMessageId());
    }
}
