package com.couplet.online.utils;

import com.couplet.common.domain.Vehicle;
import com.couplet.remote.RemoteVehicleService;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.eclipse.paho.client.mqttv3.*;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.List;
import java.util.Properties;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

/**
 * @ProjectName: five-groups-couplet
 * @Author: LiuYunHu
 * @CreateTime: 2024/4/1
 * @Description: mqtt监听者
 */

@Component
@Slf4j
public class MqttMonitor {
    /*
     * 路径
     * */
    @Value("${mqtt.server.broker}")
    private String broker;

    /*
     * 主题
     * */
    @Value("${mqtt.server.topic}")
    private String topic;

    /*
     * 客户端id
     * */
    @Value("${mqtt.server.clientId}")
    private String clientId;

    /*
     * 用户名
     * */
    @Value("${mqtt.server.username}")
    private String userName;

    /*
     * 密码
     * */
    @Value("${mqtt.server.password}")
    private String password;

    /*
     * 遗嘱消息qos
     * */
    @Value("${mqtt.server.qos}")
    private Integer qos;

    //远程调用
    @Autowired
    private RemoteVehicleService remoteVehicleService;

    //redis
    @Autowired
    private StringRedisTemplate redis;


    //Kafka生产者配置
    private static final String TOPIC_NAME = "online";
    private static final String BOOTSTRAP_SERVERS = "39.103.133.136:9092";

    //线程池，用于异步处理消息到来时的业务逻辑
    private ExecutorService executorService = Executors.newSingleThreadExecutor();

    //Kafka生产者实例化为类成员变量
    private KafkaProducer<String, String> kafkaProducer;

    //随项目启动而执行这个方法
    @PostConstruct
    public void connect() {
        log.info("mqtt监听者启动");
        try {
            MqttClient client = new MqttClient(broker, clientId, new MemoryPersistence());

            MqttConnectOptions options = new MqttConnectOptions();
            options.setUserName(userName);
            options.setPassword(password.toCharArray());

            //连接超时
            options.setConnectionTimeout(10);

            //心跳时间
            options.setKeepAliveInterval(60);

            log.info("mqtt监听者启动成功，连接到：{}", broker);

            client.connect(options);
            log.info("连接成功！");

            //设置回调
            client.setCallback(new MqttCallback() {
                @Override
                public void connectionLost(Throwable throwable) {
                    // 处理连接断开的情况
                    // 在这里执行重连操作
                    log.error("连接丢失：{}", throwable.getMessage());
                    while (!client.isConnected()) {
                        try {
                            //等待五秒后重新连接
                            Thread.sleep(5000);
                            //重新连接
                            client.reconnect();
                            log.info("重连中...");
                        } catch (InterruptedException | MqttException e) {
                            log.error("重连失败：" + e.getMessage());
                        }

                    }
                }

                @Override
                public void messageArrived(String topic, MqttMessage mqttMessage) {
                    executorService.execute(() -> processMessageArrival(topic, mqttMessage));
                }

                @Override
                public void deliveryComplete(IMqttDeliveryToken iMqttDeliveryToken) {
                    log.info("消息发送成功！");
                }
            });

            client.subscribe(topic, qos);


        } catch (MqttException e) {
            log.error("mqtt监听者启动失败，{}", e.getMessage());
//            throw new RuntimeException(e);
        }
    }


    //异步处理，mqtt消息到达后的逻辑
    private void processMessageArrival(String topic, MqttMessage mqttMessage) {
        //       log.info("消息已送达");
        //       log.info("接收消息主题：{}",topic);
        //       log.info("接收消息qos：{}", mqttMessage.getQos());

        //接收到的原始报文
        String message = new String(mqttMessage.getPayload());

        //log.info("接收消息原始内容：{}", message);

        //解析后的字符串
        String parseMsg = ParseMessageUtil.parseMsg(message);

        //拿到前17位（车辆vin码）
        String start17 = parseMsg.substring(0, 17);


        log.info("当前车辆的vin码为：" + start17);

//                    //判断缓存中是否有这个vin
//                    if (redis.hasKey("不存在的车辆VIN" + start17)) {
//
//                        //可使用RabbitMQ发送消息
//                        log.error("vin码为" + start17 + "的车辆不属于本系统！");
//
//                    } else {//如果缓存中没有存这个vin
//
//
//                    }


        //调取接口，通过vin查询车辆
        List<Vehicle> vehicles = remoteVehicleService.findByVIN(start17).getData();
        System.out.println("**************" + vehicles);


        //如果不存在这个车
        if (0 == vehicles.size()) {
            //将不属于自己系统的车辆存入缓存，便于提前进行拒绝提示
//                        redis.setCacheObject("不存在的车辆VIN" + start17, start17);
            log.error("未找到vin码为" + start17 + "的车辆信息");
        } else {
            //如果存在这个车
            Vehicle vehicle = vehicles.get(0);
            log.info("远程调用查询到的车辆数据：" + vehicle);

            //上线车辆存入redis   6秒    用于判断车辆是否下线，还要写定时器，定时查询
            redis.opsForValue().set(start17, start17, 6L, TimeUnit.SECONDS);


            log.info("vin码为" + start17 + "的车辆属于本系统，允许上线！");

            //调用上线接口，修改上线状态
            Integer i = remoteVehicleService.onOrOutLineByVIN(start17 + "," + 1);
            //上线成功
            if (0 != i) {
                log.info("上线成功！");
                try {
                    produceMessage(message);
                } catch (Exception e) {
                    log.error("发送信息异常：" + e.getMessage());
                }
            }


        }

    }

    //Kafka生产者实例化为类成员变量
    @PostConstruct
    public void initKafkaProducer() {
        Properties props = new Properties();
        props.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, BOOTSTRAP_SERVERS);
        props.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, "org.apache.kafka.common.serialization.StringSerializer");
        props.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, "org.apache.kafka.common.serialization.StringSerializer");

        kafkaProducer = new KafkaProducer<>(props);
    }


    //发送消息至Kafka
    private void produceMessage(String message) {

        try {
            //发送消息
            kafkaProducer.send(new ProducerRecord<>(TOPIC_NAME, message));

            System.out.println("发送消息：" + message);

        } catch (Exception e) {
            log.error("消息发送失败：" + e.getMessage());
        }
    }
}
